/* Evaluador de Expresiones escrito en C++
   
 Propiedad Intelectual
 Nombre:	Rafael Alberto Moreno Parra
 
 Programa:	Evaluador de expresiones en Java

 Fecha:		29 de Junio de 2005

 Explicacion:

   Sus caracteristicas principales son:
   - Evaluador NO recursivo.
   - Optimizado para ciclos.
   - Evalua funciones trigonometricas en radianes.
   - Compatible con C++ clsico
   - Hasta 100 parntesis
   - Hasta 23 variables
   - Tamao mximo de la ecuacin es de 2000 caracteres

   Forma de uso:

	 Ej:

		 public static void main(String sbParametros[])
		 {
		 	EvalExpr2 objExpresion = new EvalExpr2();
		
			String sbEcuacion = "sin(x)+cos(y)-tan(z)";
			
			//Validar la sintaxis es
			int iError = objExpresion.iChequeaSintaxis(sbEcuacion);
			if (iError != 0 )
				System.out.println("Error de sintaxis: " + iError);
				
			//Evalua la expresion
			if (iError == 0)
			{
				objExpresion.fVariable['x'-'a']=7; //Valor de X
				objExpresion.fVariable['y'-'a']=8; //Valor de Y
				objExpresion.fVariable['z'-'a']=2; //Valor de Z
				
				float fValor = objExpresion.fEvaluaEcuacion(sbEcuacion);
				System.out.println(fValor);
		
				//Evalua la expresion a alta velocidad DESPUES de usar fEvaluaEcuacion
				for (int iCont=1; iCont<=500; iCont++)
				{
					objExpresion.fVariable['x'-'a'] = iCont;
					fValor = objExpresion.fRapidoEvalua();
					System.out.println(fValor);
				}
				
			}
		}


Algoritmo:

   Por ejemplo, si tuviesemos una expresion algebraica as:
	13-4*sin(3+x)+8-(12+x*y)

   la expresin se desintegra en nodos (siempre de derecha a izquierda)

   Nodo0 = "12+x*y"             => 13-4*sin(3+x)+8-Nodo0;
   Nodo1 = "3+x"                => 13-4*sinNodo1+8-Nodo0;
   Nodo2 = "sinNodo1"           => 13-4*Nodo2+8-Nodo0;
   Nodo3 = "13-4*Nodo2+8-Nodo0"

   cada Nodo es evaluado como una expresion simple (se desintegra en operandos y operadores)
   Nodo0 ="12+x*y"
   => Nodo0a= 12 +
	  Nodo0b=  x *
	  Nodo0c=  y N
	  
Estandares:
 En el codigo, se uso el estandar Hungaro para nombrar variables y estandares aceptados Java
 Prefijo		Para
	i			integer
	d			float
	f			float
	sb			String
	c			char
*/
#include "NodoBus.h"
#include "NodoExprSimple.h"

class EvalExpr2
{
public:
	EvalExpr2(void);
	~EvalExpr2(void);

	/* 0 esta todo bien
	 * 1 es mala exponenciacin
	 * 2 es divisin por cero
	 * 3 es error en funcion
	 */
	unsigned int ERRORMATEMATICO;

	/* Defino los mtodos de la clase */
	float fCapturaEcuacion(char *sbEcuacion, float fX, float fY);
	float fCicloEvalua(float fX, float fY);
	float fEvaluaEcuacion(char *sbEcuacion);
	float fRapidoEvalua();
	void vLiberaMemoria();

	/* Almacena el valor de las variables. Mximo 26 variables. */
	float fVariable[26];

private:
	/* Almacena el valor de los acumulados. Mximo 500 parntesis. */
	float fAcumular[501];
	
	/* Variable que lleva la lista de expresiones */
	unsigned int iNumAcums;
	NodoBus *objBus; //Pasea por la lista de acums
	NodoBus *objTopeLista; //Mantiene viva la lista de acums
	bool bPrimeraExpresion; //Se usa cuando se crea la cabeza la lista de expresiones
	
	/* Variable que lleva la lista de una expresin sola sin parentesis pero que se mantiene */
	NodoExprCompleja *objComple;
	
	/* Variable que lleva la lista de una expresin sola sin parentesis pero que se destruye */
	NodoExprSimple *objNodo;
	
	/* Las funciones */
	char sbFuncion[20][10];

	/* Declaracin de mtodos privados */
	void vOptimizador(char *sbEcuacion);
	void vNuevoAcumulado(char *sbExpresion);
	float fExpresionSimple();

	/* Funciones de cadena en el mismo objeto */
	void vLeft(char *sCadenaDestino, char *sCadenaOrigen, unsigned int iNumLetras);
	void vMid(char *sCadenaDestino, char *sCadenaOrigen, unsigned int iDesde, unsigned int iHasta);
	void vMid2(char *sCadenaDestino, char *sCadenaOrigen, unsigned int iDesde);
};
