/*	Autor: Rafael Alberto Moreno Parra
	Fecha: 09 de Enero de 2009

	Objetivo: Probar el mtodo de MonteCarlo de bsqueda del mejor camino para usarlo como bsqueda de la mejor combinacin de ecuaciones para regresin simblica.
*/

#include <stdio.h>
#include <time.h>
#include <string.h>
#include "Inicializa.h"
#include "EvalExpr2.h"

#define TAMANOTABLERO 30
#define LONGITUDEXPR 100
#define TAMANOENTRADA 100
#define TOTALSIMULACION 20

//Define el arreglo unidimensional de caminos
unsigned int iCamino[TAMANOTABLERO];
	
//Define la matriz de costos
char sbMatriz[TAMANOTABLERO][TAMANOTABLERO][LONGITUDEXPR];

int main(void);
unsigned int iCalculaCostoCamino(void);
float fCalculaAproximacion(unsigned int iLongitud);
void vGeneraCaminoAzar(unsigned int iLongitud);
float fEvaluaCamino(unsigned int iLongitud);

//========== Clase de inicializacin ====
Inicializa objInicializa;
EvalExpr2 objEvalua[TAMANOTABLERO][TAMANOTABLERO];

int main()
{
	//==============================================
	//Pantalla de bienvenida
	//==============================================
	objInicializa.vPantallaIni();

	//==============================================
	//Lee el archivo de inicializacin
	//==============================================
	if (objInicializa.iLeeArchivoIni() == -1)
	{
		printf("Error leyendo el archivo de inicializacin\n");
		return 0;
	}

	//==============================================
	//Crea las tablas de probabilidades
	//==============================================
	objInicializa.vCreaTablas();

	//================================================
	//Inicia el generador de numeros pseudo-aleatorios
	//================================================
	objInicializa.vIniciaAzar();
	
	//================================================
	//Llena la matriz de ecuaciones al azar
	//================================================
	for(unsigned int iValX=0; iValX<objInicializa.stDatPG.TAMANOMATRIZ; iValX++)
		for(unsigned int iValY=0; iValY<objInicializa.stDatPG.TAMANOMATRIZ; iValY++)
		{
			objInicializa.vCreaEcuacion(sbMatriz[iValX][iValY]);
			float fValor = objEvalua[iValX][iValY].fEvaluaEcuacion(sbMatriz[iValX][iValY]);
		}

	//================================================================================
	//Va probando la longitud del camino de 2 a objInicializa.stDatPG.TAMANOMATRIZ
	//================================================================================
	unsigned int iMejoresCaminos[TAMANOTABLERO][TAMANOTABLERO];

	float fValorSimulacion[30][TAMANOTABLERO];
	for (unsigned int iNumSimulaciones=1; iNumSimulaciones<TOTALSIMULACION; iNumSimulaciones++)
	{
		for (unsigned int iLongitud=2; iLongitud<objInicializa.stDatPG.TAMANOMATRIZ; iLongitud++)
		{
			float fMejorAprox = 99999999;
			for (unsigned int iPruebasLong=1; iPruebasLong<objInicializa.stDatPG.GENERACAMINOS; iPruebasLong++)
			{
				vGeneraCaminoAzar(iLongitud);
				float fAprox = fEvaluaCamino(iLongitud);
				if (fAprox < fMejorAprox)
				{
					//Guarda la mejor aproximacin para esa longitud
					fMejorAprox = fAprox;

					//Guarda el camino hecho
					for (unsigned int iCopiar=0; iCopiar<=iLongitud; iCopiar++)
						iMejoresCaminos[iLongitud][iCopiar]=iCamino[iCopiar];
				}
			}

			//Guarda para despus graficarlo en excel
			printf("fValorSimulacion[%d][%d]=%f\n", iNumSimulaciones, iLongitud, fMejorAprox);
			fValorSimulacion[iNumSimulaciones][iLongitud]=fMejorAprox;

			printf("Longitud: %d Aproximacion: %f\n", iLongitud, fMejorAprox);
			for (unsigned int iMostrar=0; iMostrar<iLongitud; iMostrar++)
			{
				unsigned int iOrigen = iMejoresCaminos[iLongitud][iMostrar];
				unsigned int iDestino = iMejoresCaminos[iLongitud][iMostrar+1];
				printf("[%s]\n", sbMatriz[iOrigen][iDestino]);
			}
			printf("\n");
		}
	}

	//Para ser graficado por Excel
	printf("Graba en archivo\n");
	FILE *fpPlano;
	fpPlano=fopen("Estadistica.txt", "w+t");
	for (unsigned int iVertical=2; iVertical<objInicializa.stDatPG.TAMANOMATRIZ; iVertical++)
	{
		for (unsigned int iHorizontal=1; iHorizontal<TOTALSIMULACION; iHorizontal++)
			fprintf(fpPlano, "%f\t", fValorSimulacion[iHorizontal][iVertical]);
		fprintf(fpPlano, "\n");
	}
	fclose(fpPlano);

}

//Genera un camino al azar, as se repitan puntos
void vGeneraCaminoAzar(unsigned int iLongitud)
{
	//printf("Entra a vGeneraCaminoAzar\n");
	for (unsigned int iGenera=0; iGenera<iLongitud; iGenera++)
		iCamino[iGenera]=objInicializa.objAzar.genrand()%iLongitud;
	//printf("Sale de vGeneraCaminoAzar\n");
}

//Evaluar el camino y hacerle cambios usando la tcnica de bsqueda de caminos de Monte Carlo 
float fEvaluaCamino(unsigned int iLongitud)
{
	//printf("Entra a fEvaluaCamino\n");

	//Ciclo en el cual intercambia los valores del camino
	float fAproxOrig, fAproxFinal;
	for (unsigned int iPruebas=1; iPruebas<=objInicializa.stDatPG.CAMBIACAMINOS; iPruebas++)
	{
		//Calcula la aproximacin del camino original
		fAproxOrig = fCalculaAproximacion(iLongitud);
		
		//Hace un cambio al azar
		unsigned int iPosicion = objInicializa.objAzar.genrand()%iLongitud;
		unsigned int iValor = iCamino[iPosicion];
		iCamino[iPosicion] = objInicializa.objAzar.genrand()%objInicializa.stDatPG.TAMANOMATRIZ;

		//Calcula la aproximacin del nuevo camino
		fAproxFinal = fCalculaAproximacion(iLongitud);

		//Si el cambio no fue provechoso, vuelve a su posicin original
		if (fAproxFinal>fAproxOrig)
			iCamino[iPosicion]=iValor;
	}
	//printf("Sale de fEvaluaCamino\n");
	return fAproxOrig; 
}

//Calcula la aproximacin
float fCalculaAproximacion(unsigned int iLongitud)
{
	//printf("Entra a fCalculaAproximacion\n");
	unsigned int iCont;
	char sbEcuacion[LONGITUDEXPR];
	float fDiferencia;

	//Hace una copia de los valores de entrada
	float fEntrada[TAMANOENTRADA];
	for (iCont=0; iCont<objInicializa.stDatPG.NUMENTRADA; iCont++)
		fEntrada[iCont] = objInicializa.stDatPG.fENTRADA[iCont];
	
	for (int iGenera=0; iGenera<iLongitud; iGenera++)
	{
		unsigned int iOrigen = iCamino[iGenera];
		unsigned int iDestino = iCamino[iGenera+1];
				
		//strcpy(sbEcuacion, sbMatriz[iOrigen][iDestino]);
		//fDiferencia = objEvalua.fEvaluaEcuacion(sbEcuacion);
		for(iCont=0; iCont<objInicializa.stDatPG.NUMENTRADA; iCont++)
		{
			objEvalua[iOrigen][iDestino].fVariable['x'-'a'] = fEntrada[iCont];
			fEntrada[iCont] = objEvalua[iOrigen][iDestino].fRapidoEvalua();
		}
	}
	
	//objEvalua.vLiberaMemoria();

	float fAproxTotal=0;
	for(iCont=0; iCont<objInicializa.stDatPG.NUMENTRADA; iCont++)
	{
		fDiferencia = fEntrada[iCont] - objInicializa.stDatPG.fSALIDA[iCont];
		if (fDiferencia > 0)
			fAproxTotal+=fDiferencia;
		else
			fAproxTotal-=fDiferencia;		
	}
	//printf("Sale de fCalculaAproximacion\n");
	return fAproxTotal;
}

